#include "ColumnBufferArray.h"

#include <assert.h>

#include "Util.h"

namespace YARC {

/*
 * The column buffer array object inherits the clocked buffer array
 * and impelemnts the column-buffer-specific flow control, ie. the 
 * sideband received from the sideband extractor over throttle gate
 * is stored and sending a new packet is allowed if the corresponding
 * virtual channel isn't congested. The switch-external flow control
 * is virtual cut through, ie. once a packet starts transmitting it's 
 * transmitted completely irrespective of the current congestion state.
 */

Define_Module(ColumnBufferArray);

void ColumnBufferArray::initialize() {
    ClockedBufferArray::initialize();
    
    throttle = gate("throttle");
    
    isCongested[0] = 0;
    isCongested[1] = 0;
    
    WATCH_FLAG(isCongested[0]);
    WATCH_FLAG(isCongested[1]);
}

bool ColumnBufferArray::canSend() {
    int vc = getCurrentOutputVC();
    
    assert(vc == 0 || vc == 1);
        
    if (isCongested[vc]) {
        return false;
    } else {
        return true;
    }
}

bool ColumnBufferArray::canSendHeadFlit(HeadFlit *head) {
    int vc = head->getVC();
        
    assert(vc == 0 || vc == 1);

    if (isCongested[vc]) {
        return false;
    } else {
        return true;
    }
}

bool ColumnBufferArray::canSendFlit(Flit*) {
    return true;
}

bool ColumnBufferArray::canSendTailFlit(TailFlit *tail) {
    return true;
}

void ColumnBufferArray::handleMessage(cMessage *msg) {
    if (msg->getArrivalGate() == throttle) {
        Sideband *sbd = check_and_cast<Sideband*>(msg);
        copySideband(sbd, isCongested);
        delete sbd;
    } else {
        ClockedBufferArray::handleMessage(msg);
    }
}

}
