#include "DFlipFlop.h"

#include "Util.h"

#include <assert.h>

namespace YARC {

/*
 * This module serves to synchronize external ( arriving via the in gate ) events
 * with the clock defined by dffClockStart, dffClockPeriod. An external event is 
 * buffered until the next clock edge and the sent through the out gate.
 */

Define_Module(DFlipFlop);

DFlipFlop::DFlipFlop() {
    
}
 
void DFlipFlop::initialize() {
    in = gate("in");
    out = gate("out");
    
    strict = par("strict");
    WATCH(strict);
    
    lastClockT = par("dffClockStart");
    WATCH(lastClockT);
    
    clockPeriod = par("dffClockPeriod");
    WATCH(clockPeriod);
}

void DFlipFlop::handleMessage(cMessage *msg) {
    simtime_t now = simTime();

    if (strict && now <= lastClockT) {        
        assert(false);
    }
    
    while (lastClockT < now) {
        lastClockT += clockPeriod;
    }
    simtime_t delay = lastClockT - now;
        
    sendDelayed(msg, delay, out);     
}

}
