#include "IdealCrossbar.h"

#include <assert.h>

#include "Util.h"
#include "Messages_m.h"

namespace YARC {

/*
 * This module is an implementation detail of the YARC model and represents an
 * ideal crossbar. It marks an incoming message's input port with UNICAST_SRC_ATTR
 * and sends it to the wanted output port UNICAST_DST_ATTR with no delay or blocking.
 * 
 * This module is used for modeling a set of column or row connections in YARC using
 * a single simple module and star topology.
 */

Define_Module(IdealCrossbar);

void IdealCrossbar::initialize() {
    reflectGateVector(this, "ports$i",  in,  inSize,  inBase);
    reflectGateVector(this, "ports$o", out, outSize, outBase);
}

void IdealCrossbar::handleMessage(cMessage *msg) {
    int dPort = xtrLongPar(msg, UNICAST_DST_ATTR);
    assert(dPort >= 0 || dPort < outSize);
    
    int sPort = msg->getArrivalGateId() - inBase;
    addLongPar(msg, UNICAST_SRC_ATTR, sPort);
            
    send(msg, out[dPort]);     
}

}

