#include "SidebandExtractor.h"

#include "Messages_m.h"
#include "Util.h"

namespace YARC {

/*
 * This module takes the sideband ( ie. pair of start/stop flags ) from incoming
 * flits and sends it over the out gate. In the model a tile's sideband extractor
 * is connected directly to the tiles column buffers. 
 */

Define_Module(SidebandExtractor);

void SidebandExtractor::initialize() {
    in  = gate("in");
    out = gate("out");
    sidebandOut = gate("sideband");
}

void SidebandExtractor::handleMessage(cMessage *msg) {
    Flit *flit = check_and_cast<Flit*>(msg);
    
    Sideband *sideband = new Sideband();
    copySideband(flit, sideband);
    
    send(sideband, sidebandOut);
    
    IdleFlit *idle = dynamic_cast<IdleFlit*>(msg);
    if (idle != NULL) {
        delete idle;
    } else {
        send(msg, out);
    }
}

}
