#include "TagDemux.h"

#include <string.h>
#include <assert.h>

#include "Util.h"

namespace YARC {

/*
 * This module is an implementation detail of the YARC model. It's intended to
 * be used together with the TagMux module. It consumes messages from the single
 * in gate, reads and removes the tag and sends the message over a different out gate
 * depending on the tag. 
 */

Define_Module(TagDemux);

void TagDemux::initialize() {
    in = gate("in");
    reflectGateVector(this, "out",  out,  size,  base);
    
    const char *tagsConcat = par("tags");
    int tagCount = tokenizeString(tagsConcat, tags);
    assert(tagCount == size);
    
    WATCH(size);
    for (int i=0; i < size; i++) {
        const int BUFF_SIZE = 100;
        char *buff = newBuff(BUFF_SIZE);        
        
        snprintf(buff, BUFF_SIZE, "tags[%d]", i);        
        createWatch(buff, tags[i]);
    }
}

void TagDemux::handleMessage(cMessage *msg) {
    const char *tag = msg->par(MULDEX_TAG_ATTR);
    
    for (int i=0; i < size; i++) {
        if (strcmp(tag, tags[i]) == 0) {
            delete msg->removeObject(MULDEX_TAG_ATTR);
            
            cGate *g = out[i];
            send(msg, g);
            
            return;
        }
    }
        
    assert(false);
}

}
