#include "TagMux.h"

#include <assert.h>
#include <stdio.h>

#include "Util.h"

namespace YARC {

/*
 * This module is an implementation detail of the YARC model. It takes multiple
 * incoming message streams over different in gates. Each message is tagged with
 * an in-gate specific tag and sent over the single out gate. The demux module
 * does the inverse splitting.   
 */

Define_Module(TagMux);

void TagMux::initialize() {
    reflectGateVector(this, "in",  in,  size,  base);
    out = gate("out");
    
    const char *tagsConcat = par("tags");
    int tagCount = tokenizeString(tagsConcat, tags);
    assert(tagCount == size);
    
    WATCH(size);
    for (int i=0; i < size; i++) {
        const int BUFF_SIZE = 100;
        char *buff = newBuff(BUFF_SIZE);       
        
        snprintf(buff, BUFF_SIZE, "tags[%d]", i);        
        createWatch(buff, tags[i]);
    }
}

void TagMux::handleMessage(cMessage *msg) {
    for (int i=0; i < size; i++) {
        if (msg->getArrivalGate() == in[i]) {
            addCStringPar(msg, MULDEX_TAG_ATTR, tags[i]);
            send(msg, out);
            
            return;
        }
    }
    
    assert(false);
}

}
